use argh::FromArgs;

#[derive(FromArgs)]
/// Control your herd
pub struct SheepctlArgs {
    #[argh(subcommand)]
    pub command: SheepctlCommand,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum SheepctlCommand {
    Login(LoginCommand),
    Device(DeviceCommand),
    Shell(ShellCommand),
    Query(QueryCommand),
}

#[derive(FromArgs, PartialEq, Debug)]
/// Login to a homeserver
#[argh(subcommand, name = "login")]
pub struct LoginCommand {
    #[argh(positional)]
    /// homeserver
    pub home: String,

    #[argh(positional)]
    /// username
    pub username: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "device")]
/// Commands for devices
pub struct DeviceCommand {
    #[argh(subcommand)]
    pub command: DeviceCommands,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand)]
pub enum DeviceCommands {
    List(ListDevicesCommand),
}

#[derive(FromArgs, PartialEq, Debug)]
/// List devices
#[argh(subcommand, name = "ls")]
pub struct ListDevicesCommand {
    #[argh(switch)]
    /// only show online devices
    pub online: bool,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "shell")]
/// Enter interactive shell
pub struct ShellCommand {
    #[argh(positional)]
    /// device ID
    pub device: String,
}

#[derive(FromArgs, PartialEq, Debug)]
#[argh(subcommand, name = "query")]
/// Run an osquery
pub struct QueryCommand {
    #[argh(positional)]
    /// device ID
    pub device: String,
    #[argh(positional)]
    /// query
    pub query: String,
}
