use owl::{Deserialize, Serialize};

use crate::api::JoinResponse;

/// Client agent configuration at `/etc/sheepd/config.toml`
#[derive(Serialize, Deserialize, Debug)]
pub struct AgentConfig {
    /// Homeserver Domain
    pub home: String,
    /// Homeserver Token
    pub token: String,
    /// MQTT endpoint
    pub mqtt: String,
    /// Server Public Encryption Key
    pub server_age: String,
    /// Server Public Sign Key
    pub server_sign: String,
}

impl AgentConfig {
    pub fn try_load() -> Option<Self> {
        toml::from_str(&std::fs::read_to_string("/etc/sheepd/config.toml").ok()?).ok()
    }

    pub fn new(home: &str, join: JoinResponse) -> Self {
        Self {
            token: join.token,
            mqtt: join.mqtt,
            home: home.to_string(),
            server_age: join.identity.0,
            server_sign: join.identity.1,
        }
    }
}
